@@ -1,14 +1,16 @@
 package Mojo::Webqq;
 use strict;
-$Mojo::Webqq::VERSION = "1.7.1";
+use Carp ();
+$Mojo::Webqq::VERSION = "1.7.2";
 use base qw(Mojo::Base);
 use Mojo::Webqq::Log;
 use Mojo::Webqq::Cache;
+use Mojo::Webqq::Counter;
 sub has { Mojo::Base::attr(__PACKAGE__, @_) };
 use Time::HiRes qw(gettimeofday);
 use POSIX;
 use File::Spec ();
-use base qw(Mojo::EventEmitter Mojo::Webqq::Base Mojo::Webqq::Model Mojo::Webqq::Client Mojo::Webqq::Message Mojo::Webqq::Plugin);
+use base qw(Mojo::EventEmitter Mojo::Webqq::Base Mojo::Webqq::Model Mojo::Webqq::Client Mojo::Webqq::Message Mojo::Webqq::Plugin Mojo::Webqq::Util);
 
 has qq                  => undef;
 has pwd                 => undef;
@@ -21,8 +23,18 @@ has log_level           => 'info';     #debug|info|warn|error|fatal
 has log_path            => undef;
 has log_encoding        => undef;      #utf8|gbk|...
 has email               => undef;
-has encrypt_method      => "perl";     #perl|js
 
+has is_init_friend         => 1;                            #是否在首次登录时初始化好友信息
+has is_init_group          => 1;                            #是否在首次登录时初始化群组信息
+has is_init_discuss        => 1;                            #是否在首次登录时初始化讨论组信息
+has is_init_recent         => 0;                            #是否在首次登录时初始化最近联系人信息
+
+has is_update_user          => 0;                            #是否定期更新个人信息
+has is_update_group         => 1;                            #是否定期更新群组信息
+has is_update_friend        => 1;                            #是否定期更新好友信息
+has is_update_discuss       => 1;                            #是否定期更新讨论组信息
+
+has encrypt_method      => "perl";     #perl|js
 has tmpdir              => sub {File::Spec->tmpdir();};
 has pic_dir             => sub {$_[0]->tmpdir};
 has cookie_dir          => sub{return $_[0]->tmpdir;};
@@ -142,10 +154,20 @@ sub new {
     my $class = shift;
     my $self  = $class->Mojo::Base::new(@_);
     #$ENV{MOJO_USERAGENT_DEBUG} = $self->{ua_debug};
+    $self->info("当前正在使用 Mojo-Webqq v" . $self->version);
     if(not defined $self->{qq}){
         $self->fatal("客户端初始化缺少qq参数");
         $self->exit();
     }
+    $self->ioloop->reactor->on(error=>sub{
+        my ($reactor, $err) = @_;
+        $self->error("reactor error: " . Carp::longmess($err));
+    });
+    $SIG{__WARN__} = sub{$self->warn(Carp::longmess @_);};
+    $self->on(error=>sub{
+        my ($self, $err) = @_;
+        $self->error(Carp::longmess($err));
+    });
     $self->on(qrcode_expire=>sub{
         my($self) = @_;
         my $count = $self->qrcode_count;
@@ -167,19 +189,36 @@ sub new {
             $self->relogin();
         }
     });
+    $self->on(new_group=>sub{
+        my($self,$group)=@_;
+        $self->update_group($group,is_blocking=>1,is_update_group_ext=>1,is_update_group_member_ext=>1);
+    });
+
+    $self->on(new_group_member=>sub{
+        my($self,$member)=@_;
+        $member->group->update_group_member_ext(is_blocking=>1);
+    });
+    $self->on(new_friend=>sub{
+        my($self,$friend)=@_;
+        $self->update_friend_ext(is_blocking=>1);
+    });
+    $Mojo::Webqq::Client::CLIENT_COUNT++;
     $self;
 }
 
 sub friends{
     my $self = shift;
+    $self->update_friend() if @{$self->friend} == 0;
     return @{$self->friend};
 }
 sub groups{
     my $self = shift;
+    $self->update_group() if @{$self->group} == 0;
     return @{$self->group};
 }
 sub discusss{
     my $self = shift;
+    $self->update_discuss() if @{$self->discuss} == 0;
     return @{$self->discuss};
 }
 sub recents{
