@@ -37,18 +37,21 @@ sub each_group_member{
     my $callback = shift;
     $self->{_client}->die("参数必须是函数引用") if ref $callback ne "CODE";
     return if ref $self->member ne "ARRAY";
+    $self->{_client}->update_group($self) if $self->is_empty;
     for(@{$self->member}){
         $callback->($self->{_client},$_); 
     }
 }
 sub members {
     my $self = shift;
+    $self->{_client}->update_group($self) if $self->is_empty;
     return @{$self->member};
 }
 sub search_group_member{
     my $self = shift;
     my %p = @_;
     return if 0 == grep {defined $p{$_}} keys %p;
+    $self->{_client}->update_group($self) if $self->is_empty;
     if(wantarray){
         return grep {my $m = $_;(first {$p{$_} ne $m->$_} grep {defined $p{$_}} keys %p) ? 0 : 1;} @{$self->member};
     }
@@ -94,10 +97,18 @@ sub is_empty{
     return !(ref($self->member) eq "ARRAY"?0+@{$self->member}:0);
 }
 
+sub update_group_member_ext {
+    my $self = shift;
+    $self->{_client}->update_group_member_ext($self,@_);
+}
+sub update_group_member {
+    my $self = shift;
+    $self->{_client}->update_group_member($self,@_);
+}
 sub update{
     my $self = shift;
     my $hash = shift;
-    for(grep {substr($_,0,1) ne "_"} keys %$self){
+    for(grep {substr($_,0,1) ne "_"} keys %$hash){
         if($_ eq "member" and ref $hash->{member} eq "ARRAY"){
             next if not @{$hash->{member}};
             my @member = map {ref $_ eq "Mojo::Webqq::Group::Member"?$_:$self->{_client}->new_group_member($_)} @{$hash->{member}};
